require_dependency "fly_admin/application_controller"

module FlyAdmin
  class UploadsController < ApplicationController
    # needed for daemon_enabled?
    include SettingsHelper
    include ::VideoModule
    def index
      @categories = Category.all.each {|c| c.json_name = JSON.parse(c.name)["ru"]}
    end

    def upload
      redirect_to upload_path, alert: "Пожалуйста укажите сезон " and return unless params["season_id"].present? 
      redirect_to upload_path, alert: "Пожалуйста укажите файлы для загрузки" and return if params["nodes"] == "[]"
      group_files_for_uploading
      if daemon_enabled?
        flash[:notice] = "Конвертация запущена!"
      else
        flash[:alert] = "Конвертация запущена, но у вас выключен демон в настройках!( после запуска демона конвертация начнется автоматически )"
      end
      redirect_to videos_path(season_id: params["season_id"], after_upload: 'yes')
    end

    def files
      @path = SiteConfig["torrents_path"]
      
      if params[:li_attr]
        url = params[:li_attr][:url].shellescape 
      else
        url = ""
      end
      
      url = params[:id] == '#' ? @path : @path + url
      nodes = Dir["#{url}/*"].map {|p| p.split(@path).last }
      nodes = tree_js_format(nodes)
      render status: 404, text: 'Нет файлов!' and return if nodes.empty?
      render json: nodes
    end

    private

    def short_path(path)
      if path.count("/") == 1
        # if root
        path.delete "/"
      else
        # if nested
        path.split("/").last
      end
    end

    def has_children(node)
      Dir[@path + node.shellescape + "/*"].any?
    end

    def tree_js_format(nodes)
      nodes.map do |node|
        # if file
        if video_file?(node)
          { text: short_path(node), icon: 'jstree-file', li_attr: {url: node, type: :file}}
        # if folder
        else
          if has_children(node)
            { text: short_path(node), children: true, hide_checkbox: true , disabled: true, li_attr: {url: node, type: :folder}}
          else
            # if empty folder
            nil
          end
        end
      end.compact
    end

    def get_category_id_from_params
      if new_category?
        Category.create(:name => params['category_name']).to_param
      else
        params['category_id']
      end
    end

    def group_files_for_uploading
      category_id = params['category_id']
      nodes = JSON.parse(params[:nodes])
      
      season_id = params[:season_id]
      path = SiteConfig["torrents_path"]
      ConvertVideoJob.perform_later(nodes: nodes, season_id: season_id, path: path, category_id: category_id)
    end

    def new_category?
      params['category_name']
    end

    def redirect_back_if_errors
    end

  end
end
