module FlyAdmin
  class ConvertVideoJob < ActiveJob::Base
    include ::VideoModule
    queue_as :default

    def perform(options)
      nodes = options[:nodes]
      season_id = options[:season_id]
      path = options[:path]
      category_id = options[:category_id]
      folders, files = nodes.partition { |n| n['li_attr']['type'] == 'folder' }

      files_for_convert = []

      files.select {|file| video_file?(file["li_attr"]["url"]) }.each do |node|
        options = {
          file_path: node["li_attr"]["url"],
          category_id: category_id,
          title: node["name"],
          season_id: season_id
        }
        files_for_convert << options
      end

      folders.each do |node|
        node_path = node["li_attr"]["url"]
        files = Dir["#{path + node_path.shellescape}/**/*.{mp4,mp3,aaf,3gp,gif,asf,avchd,avi,cam,dat,dsh,flv,mpeg,mpeg-1,mpeg-2,fla,flr,sol,m4v,mkv,wrap,mng,mov,mpg,mpe,mxf,nsv,ogg,svi,smi,wmv}"]
        files.each do |file|
          file_path = file.split(path).last # remove root from path
          title = file_path.split("/").last
          options = {
            file_path: file_path,
            category_id: category_id,
            title: title,
            season_id: season_id
          }
          files_for_convert << options
        end
      end

      # Convert video stuff
      files_for_convert.uniq.each { |file| process_video(file) }
    end

    # making thumbnail, moving file
    def process_video(video)
      title, file_path, season_id = video[:title], video[:file_path], video[:season_id]
      filename = file_path.split("/").last.gsub(/[^0-9A-Za-z.\-]/, '_')

      v = Video.create!(:title => title, :content => filename, :state => Video::BEFORE_SPAWNLING, :source_path => file_path, :uuid => SecureRandom.uuid)
      dir = File.dirname(v.content_path)
      FileUtils.mkdir_p(dir)

      v.set_duration
      v.delay.create_thumbnail

      season = Season.find(season_id)
      season.videos << v
      CONVERTATION_LOG.info "Convert Video id : #{v.id}"

      v.update_column(:state, Video::PENDING_STATE)
      # For debug only
    end
  end
end
