module FlyAdmin
  class InstallGenerator < Rails::Generators::Base
    source_root File.expand_path('../templates', __FILE__)
    desc 'Copy FlyAdmin initializer'
    FLY_ADMIN_PATH = Gem.loaded_specs['fly_admin'].full_gem_path
    
    class_option 'with-admin-user', type: :boolean, desc: 'Create admin user'
    class_option 'with-migrating-footer', type: :boolean, desc: 'Migrate footers to fly_admin for old paysites'

    def copy_fly_admin_initializer
      copy_file "fly_admin_initializer.rb", "config/initializers/fly_admin.rb"
    end

    # Delete old files in main app
    def delete_page_cache
      remove_file('lib/page_cache.rb')
    end

    def delete_initializer_page_cache
      remove_file('config/initializers/page_cache')
    end

    def add_fly_admin_route
      if engine_not_mounted?
        route "mount FlyAdmin::Engine, at: '/fly_admin'"
      end
    end

    # Create Admin user
    def bootstrap_admin
      if options['with-admin-user']
        if admin_not_exists? 
          rake 'fly_admin:bootstrap:admin'
        end
      end
    end

    # Create records on models: Country, Paysite, which needed for displaying footer
    def bootstrap_countries
      if countries_not_bootstraped?
        rake 'fly_admin:bootstrap:countries'
      end
    end

    def bootstrap_paysites
      if paysites_not_bootstraped?
        rake 'fly_admin:bootstrap:paysites'
      end
    end

    # Migrate existing footers
    def migrate_footers
      if options['with-migrating-footer']
        rake 'fly_admin:migrate:footers'
      end
    end

    def replace_state_machine_with_state_machines
      gemfile_path = "#{Rails.root}/Gemfile"
      gemfile = File.read(gemfile_path)
      new_contents = gemfile.gsub(/gem "state_machine", :git => '.*'/, "gem 'state_machines-activerecord'")
      new_contents.gsub!(/# using git.*295/, "")
      File.open(gemfile_path, 'w') {|f| f.write(new_contents) }  
      ap "Running bundle install", :color => { :string => :green }
      `bundle install`
    end

    def replace_connection_api
      remove_file('lib/connection_api.rb')
      controller_path = "#{Rails.root}/app/controllers/application_controller.rb"
      controller_contents = File.read(controller_path)
      controller_contents.gsub!(/ ConnectionApi/, " FlyAdmin::Imbs::ConnectionApi")
      File.open(controller_path, 'w') { |f| f.write(controller_contents) }
    end

    private
    def engine_not_mounted?
      File.open("#{Rails.root}/config/routes.rb").each_line do |line|
        if (line.include? "mount FlyAdmin::Engine")
          return false
        end
      end
      return true
    end

    def countries_not_bootstraped?
      FlyAdmin::Country.first.nil?
    end

    def paysites_not_bootstraped?
      FlyAdmin::Paysite.first.nil?
    end

    def admin_not_exists?
      User.find_by_login('admin').nil?
    end
  end
end
